"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandNames = exports.rcOptionsTypes = void 0;
exports.cliOptionsTypes = cliOptionsTypes;
exports.help = help;
exports.handler = handler;
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const common_cli_options_help_1 = require("@pnpm/common-cli-options-help");
const config_1 = require("@pnpm/config");
const types_1 = require("@pnpm/types");
const error_1 = require("@pnpm/error");
const get_context_1 = require("@pnpm/get-context");
const workspace_find_packages_1 = require("@pnpm/workspace.find-packages");
const logger_1 = require("@pnpm/logger");
const pick_1 = __importDefault(require("ramda/src/pick"));
const partition_1 = __importDefault(require("ramda/src/partition"));
const render_help_1 = __importDefault(require("render-help"));
const createProjectManifestWriter_1 = require("./createProjectManifestWriter");
const getSaveType_1 = require("./getSaveType");
const install = __importStar(require("./install"));
const normalize_path_1 = __importDefault(require("normalize-path"));
// @ts-expect-error
const isWindows = process.platform === 'win32' || global['FAKE_WINDOWS'];
const isFilespec = isWindows ? /^(?:[.]|~[/]|[/\\]|[a-zA-Z]:)/ : /^(?:[.]|~[/]|[/]|[a-zA-Z]:)/;
exports.rcOptionsTypes = cliOptionsTypes;
function cliOptionsTypes() {
    return (0, pick_1.default)([
        'global-dir',
        'global',
        'only',
        'package-import-method',
        'production',
        'registry',
        'reporter',
        'save-dev',
        'save-exact',
        'save-optional',
        'save-prefix',
        'unsafe-perm',
    ], config_1.types);
}
exports.commandNames = ['link', 'ln'];
function help() {
    return (0, render_help_1.default)({
        aliases: ['ln'],
        descriptionLists: [
            {
                title: 'Options',
                list: common_cli_options_help_1.UNIVERSAL_OPTIONS,
            },
        ],
        url: (0, cli_utils_1.docsUrl)('link'),
        usages: [
            'pnpm link <dir|pkg name>',
            'pnpm link',
        ],
    });
}
async function checkPeerDeps(linkCwdDir, opts) {
    const { manifest } = await (0, cli_utils_1.tryReadProjectManifest)(linkCwdDir, opts);
    if (manifest?.peerDependencies && Object.keys(manifest.peerDependencies).length > 0) {
        const packageName = manifest.name ?? path_1.default.basename(linkCwdDir); // Assuming the name property exists in newManifest
        const peerDeps = Object.entries(manifest.peerDependencies)
            .map(([key, value]) => `  - ${key}@${String(value)}`)
            .join(', ');
        logger_1.logger.warn({
            message: `The package ${packageName}, which you have just pnpm linked, has the following peerDependencies specified in its package.json:

${peerDeps}

The linked in dependency will not resolve the peer dependencies from the target node_modules.
This might cause issues in your project. To resolve this, you may use the "file:" protocol to reference the local dependency.`,
            prefix: opts.dir,
        });
    }
}
async function handler(opts, params) {
    let workspacePackagesArr;
    let workspacePackages;
    if (opts.workspaceDir) {
        workspacePackagesArr = await (0, workspace_find_packages_1.findWorkspacePackages)(opts.workspaceDir, {
            ...opts,
            patterns: opts.workspacePackagePatterns,
        });
        workspacePackages = (0, get_context_1.arrayOfWorkspacePackagesToMap)(workspacePackagesArr);
    }
    else {
        workspacePackages = new Map();
    }
    const linkOpts = Object.assign(opts, {
        targetDependenciesField: (0, getSaveType_1.getSaveType)(opts),
        workspacePackages,
        binsDir: opts.bin,
    });
    if (opts.cliOptions?.global && !opts.bin) {
        throw new error_1.PnpmError('NO_GLOBAL_BIN_DIR', 'Unable to find the global bin directory', {
            hint: 'Run "pnpm setup" to create it automatically, or set the global-bin-dir setting, or the PNPM_HOME env variable. The global bin directory should be in the PATH.',
        });
    }
    const writeProjectManifest = await (0, createProjectManifestWriter_1.createProjectManifestWriter)(opts.rootProjectManifestDir);
    // pnpm link
    if ((params == null) || (params.length === 0)) {
        const cwd = process.cwd();
        if (path_1.default.relative(linkOpts.dir, cwd) === '') {
            throw new error_1.PnpmError('LINK_BAD_PARAMS', 'You must provide a parameter');
        }
        await checkPeerDeps(cwd, opts);
        const newManifest = opts.rootProjectManifest ?? {};
        await addLinkToManifest(opts, newManifest, cwd, opts.rootProjectManifestDir);
        await writeProjectManifest(newManifest);
        await install.handler({
            ...linkOpts,
            frozenLockfileIfExists: false,
            rootProjectManifest: newManifest,
        });
        return;
    }
    const [pkgPaths, pkgNames] = (0, partition_1.default)((inp) => isFilespec.test(inp), params);
    pkgNames.forEach((pkgName) => pkgPaths.push(path_1.default.join(opts.globalPkgDir, 'node_modules', pkgName)));
    const newManifest = opts.rootProjectManifest ?? {};
    await Promise.all(pkgPaths.map(async (dir) => {
        await addLinkToManifest(opts, newManifest, dir, opts.rootProjectManifestDir);
        await checkPeerDeps(dir, opts);
    }));
    await writeProjectManifest(newManifest);
    await install.handler({
        ...linkOpts,
        frozenLockfileIfExists: false,
        rootProjectManifest: newManifest,
    });
}
async function addLinkToManifest(opts, manifest, linkedDepDir, manifestDir) {
    if (!manifest.pnpm) {
        manifest.pnpm = {
            overrides: {},
        };
    }
    else if (!manifest.pnpm.overrides) {
        manifest.pnpm.overrides = {};
    }
    const { manifest: linkedManifest } = await (0, cli_utils_1.tryReadProjectManifest)(linkedDepDir, opts);
    const linkedPkgName = linkedManifest?.name ?? path_1.default.basename(linkedDepDir);
    const linkedPkgSpec = `link:${(0, normalize_path_1.default)(path_1.default.relative(manifestDir, linkedDepDir))}`;
    manifest.pnpm.overrides[linkedPkgName] = linkedPkgSpec;
    if (types_1.DEPENDENCIES_FIELDS.every((depField) => manifest[depField]?.[linkedPkgName] == null)) {
        manifest.dependencies = manifest.dependencies ?? {};
        manifest.dependencies[linkedPkgName] = linkedPkgSpec;
    }
}
//# sourceMappingURL=link.js.map